
# DO source /esarchive/scratch/jramon/GitLab_jramon/auto-s2s/MODULES before opening the R session

setwd("/esarchive/scratch/jramon/GitLab_jramon/auto-s2s/")
source("modules/Loading/Loading.R")
source("modules/Downscaling/Downscaling.R")
source("modules/Calibration/Calibration.R")
source("modules/Skill/Skill.R")
source("modules/Saving/Saving.R")
source("modules/Visualization/Visualization.R")

# use this development to run the analogs function. Also change the Interpolation.R script to 
# not load CDORemap from s2dv.
library(easyNCDF)
library(ncdf4)
library(ClimProjDiags)
source('https://earth.bsc.es/gitlab/es/s2dv/-/raw/master/R/Utils.R')
source('https://earth.bsc.es/gitlab/es/s2dv/-/raw/master/R/CDORemap.R')

# Loop all downscaling methods and types
#types <- c('int', 'intbc', 'intlr', 'analogs', 'logreg') 
#int_methods <- c('con', 'bil', 'bic', 'nn', 'con2')
#bc_methods <- c('simple_bias', 'calibration', 'quantile_mapping')
#lr_methods <- c('basic', 'large_scale', '4nn')
#logreg_methods <- c('ens_mean', 'ens_mean_sd', 'sorted_members')

# Read recipe
recipe_file <- "modules/Loading/testing_recipes/recipe_system5c3s-tas_downscaling.yml"
recipe <- prepare_outputs(recipe_file)

# Load datasets
data <- load_datasets(recipe)

# Modify the recipe manually
recipe$Analysis$Workflow$Downscaling$type           <- 'analogs'
recipe$Analysis$Workflow$Downscaling$int_method     <- NULL # con, bil, bic, nn, con2
recipe$Analysis$Workflow$Downscaling$bc_method      <- NULL
recipe$Analysis$Workflow$Downscaling$lr_method      <- NULL
recipe$Analysis$Workflow$Downscaling$log_reg_method <- NULL

# Downscale datasets
downscaled_data <- downscale_datasets(recipe, data)

# Compute skill
# NOTE: LogReg method does not have member dimension because it provides directly the 
# predicted categories
# The following line should work only for Interpolation, Intbc, Intlr, Analogs
#stopifnot(recipe$Analysis$Workflow$Downscaling$type %in% c('int', 'intbc', 'intlr', 'analogs'))
#skill_metrics <- compute_skill_metrics(recipe, downscaled_data$hcst$exp, downscaled_data$hcst$obs)

# Save downscaled data to later intercompare them
saveRDS(downscaled_data, file = paste0('/esarchive/scratch/jramon/downscaling/data/FOCUS/', 
        recipe$Analysis$Workflow$Downscaling$type, '-',
        recipe$Analysis$Workflow$Downscaling$int_method, '-',
        recipe$Analysis$Workflow$Downscaling$bc_method, '-',
        recipe$Analysis$Workflow$Downscaling$lr_method, '-',
        recipe$Analysis$Workflow$Downscaling$log_reg_method, '.RDS'))

# Plot downscaled data
plot_data(recipe, data = NULL, downscaled_data = downscaled_data)

# Plot skill metrics
my_data <- list()
my_data$hcst <- downscaled_data$hcst$exp
plot_data(recipe, data = my_data, skill_metrics = skill_metrics)

# Plot Taylor diagram intercomparison
source('/esarchive/scratch/jramon/GitLab_jramon/nc2ts/taylor_diagram.R')
rds_files <- list.files(path = '/esarchive/scratch/jramon/downscaling/data/FOCUS', full.names = T)
down_data <- list()
methods <- c(list.files(path = '/esarchive/scratch/jramon/downscaling/data/FOCUS'))
for (nfile in seq(rds_files)) {

  # to plot all points in the same Taylor diagram
  flag <- ifelse(nfile == 1, F, T)

  down_data[[nfile]] <- readRDS(rds_files[[nfile]])

  #compute model ensemble mean
  ens_mean <- Apply(down_data[[nfile]]$hcst$exp$data, 'ensemble', mean)$output1
 
  # remove ensemble dimension in obs
  obs <- Subset(down_data[[nfile]]$hcst$obs$data, along = 'ensemble', indices = 1, drop = 'selected')
  
  ens_mean <- .reorder_dims(arr_ref = obs, arr_to_reorder = ens_mean)

  plot <- taylor.diagram(ref = as.vector(obs), model = as.vector(ens_mean), normalize = TRUE, 
                         add = flag, pch = nfile, sd.arcs = TRUE, maxsd = 1.5)

}

legend(1.3, 1.5, cex = 0.8, pt.cex = 1, legend = methods, pch = 1:length(rds_files), ncol = 1)

##################################
# EXAMPLE FOR SEASONAL CALIBRATION
##################################
#data <- load_datasets(recipe)
# Calibrate datasets
#calibrated_data <- calibrate_datasets(recipe, data)
# Compute skill metrics
#skill_metrics <- compute_skill_metrics(recipe, calibrated_data$hcst, data$obs)
# Compute percentiles and probability bins
#probabilities <- compute_probabilities(recipe, calibrated_data$hcst)
# Export all data to netCDF
#save_data(recipe, data, calibrated_data, skill_metrics, probabilities)
# Plot data
#plot_data(recipe, data, calibrated_data, skill_metrics, probabilities,
#          significance = T)



